@@ -23,10 +23,11 @@
 #include <qplatformdefs.h>
 #include <stdio.h>
 
-#include <QtCore/QFile>
-#include <QtCore/QStandardPaths>
-#include <QtCore/QTimer>
-#include <QtCore/QProcess>
+#include <QFile>
+#include <QStandardPaths>
+#include <QTimer>
+#include <QProcess>
+#include <QElapsedTimer>
 
 #include <QDBusConnection>
 #include <klocalizedstring.h>
@@ -99,11 +100,12 @@ static bool forkSlaves()
         }
 
 #ifdef Q_OS_UNIX
-        // fallback: if there's an klauncher process owned by a different user: still fork
         if (!fork) {
             // check the UID of klauncher
             QDBusReply<uint> reply = QDBusConnection::sessionBus().interface()->serviceUid(klauncher()->service());
-            if (reply.isValid() && getuid() != reply) {
+            // if reply is not valid, fork, most likely klauncher can not be run or is not installed
+            // fallback: if there's an klauncher process owned by a different user: still fork
+            if (!reply.isValid() || getuid() != reply) {
                 fork = true;
             }
         }
@@ -123,7 +125,7 @@ namespace KIO
 class SlavePrivate: public SlaveInterfacePrivate
 {
 public:
-    SlavePrivate(const QString &protocol) :
+    explicit SlavePrivate(const QString &protocol) :
         m_protocol(protocol),
         m_slaveProtocol(protocol),
         slaveconnserver(new KIO::ConnectionServer),
@@ -134,14 +136,13 @@ public:
         dead(false),
         m_refCount(1)
     {
-        contact_started = QDateTime::currentDateTime();
-        m_idleSince = QDateTime();
+        contact_started.start();
         slaveconnserver->listenForRemote();
         if (!slaveconnserver->isListening()) {
             qCWarning(KIO_CORE) << "KIO Connection server not listening, could not connect";
         }
     }
-    ~SlavePrivate()
+    ~SlavePrivate() override
     {
         delete slaveconnserver;
     }
@@ -157,8 +158,8 @@ public:
     quint16 m_port;
     bool contacted;
     bool dead;
-    QDateTime contact_started;
-    QDateTime m_idleSince;
+    QElapsedTimer contact_started;
+    QElapsedTimer m_idleSince;
     int m_refCount;
 };
 }
@@ -170,7 +171,7 @@ void Slave::accept()
     d->slaveconnserver->deleteLater();
     d->slaveconnserver = nullptr;
 
-    connect(d->connection, SIGNAL(readyRead()), SLOT(gotInput()));
+    connect(d->connection, &Connection::readyRead, this, &Slave::gotInput);
 }
 
 void Slave::timeout()
@@ -186,10 +187,10 @@ void Slave::timeout()
     /*qDebug() << "slave failed to connect to application pid=" << d->m_pid
                  << " protocol=" << d->m_protocol;*/
     if (d->m_pid && KIOPrivate::isProcessAlive(d->m_pid)) {
-        int delta_t = d->contact_started.secsTo(QDateTime::currentDateTime());
+        int delta_t = d->contact_started.elapsed() / 1000;
         //qDebug() << "slave is slow... pid=" << d->m_pid << " t=" << delta_t;
         if (delta_t < SLAVE_CONNECTION_TIMEOUT_MAX) {
-            QTimer::singleShot(1000 * SLAVE_CONNECTION_TIMEOUT_MIN, this, SLOT(timeout()));
+            QTimer::singleShot(1000 * SLAVE_CONNECTION_TIMEOUT_MIN, this, &Slave::timeout);
             return;
         }
     }
@@ -198,7 +199,7 @@ void Slave::timeout()
     d->dead = true;
     QString arg = d->m_protocol;
     if (!d->m_host.isEmpty()) {
-        arg += "://" + d->m_host;
+        arg += QLatin1String("://") + d->m_host;
     }
     //qDebug() << "slave died pid = " << d->m_pid;
 
@@ -217,7 +218,7 @@ Slave::Slave(const QString &protocol, QObject *parent)
     Q_D(Slave);
     d->slaveconnserver->setParent(this);
     d->connection = new Connection(this);
-    connect(d->slaveconnserver, SIGNAL(newConnection()), SLOT(accept()));
+    connect(d->slaveconnserver, &ConnectionServer::newConnection, this, &Slave::accept);
 }
 
 Slave::~Slave()
@@ -271,7 +272,7 @@ QString Slave::passwd()
 void Slave::setIdle()
 {
     Q_D(Slave);
-    d->m_idleSince = QDateTime::currentDateTime();
+    d->m_idleSince.start();
 }
 
 bool Slave::isConnected()
@@ -297,19 +298,25 @@ void Slave::deref()
     Q_D(Slave);
     d->m_refCount--;
     if (!d->m_refCount) {
-        d->connection->disconnect(this);
-        this->disconnect();
-        deleteLater();
+        aboutToDelete();
+        delete this; // yes it reads funny, but it's too late for a deleteLater() here, no event loop anymore
     }
 }
 
+void Slave::aboutToDelete()
+{
+    Q_D(Slave);
+    d->connection->disconnect(this);
+    this->disconnect();
+}
+
 int Slave::idleTime()
 {
     Q_D(Slave);
-    if (d->m_idleSince.isNull()) {
+    if (!d->m_idleSince.isValid()) {
         return 0;
     }
-    return d->m_idleSince.secsTo(QDateTime::currentDateTime());
+    return d->m_idleSince.elapsed() / 1000;
 }
 
 void Slave::setPID(qint64 pid)
@@ -401,7 +408,7 @@ void Slave::gotInput()
         d->dead = true;
         QString arg = d->m_protocol;
         if (!d->m_host.isEmpty()) {
-            arg += "://" + d->m_host;
+            arg += QLatin1String("://") + d->m_host;
         }
         //qDebug() << "slave died pid = " << d->m_pid;
         // Tell the job about the problem.
@@ -417,8 +424,8 @@ void Slave::kill()
 {
     Q_D(Slave);
     d->dead = true; // OO can be such simple.
-    /*qDebug() << "killing slave pid" << d->m_pid
-                 << "(" << QString(d->m_protocol) + "://" + d->m_host << ")";*/
+    //qDebug() << "killing slave pid" << d->m_pid
+    //         << "(" << d->m_protocol + QLatin1String("://") + d->m_host << ")";
     if (d->m_pid) {
         KIOPrivate::sendTerminateSignal(d->m_pid);
         d->m_pid = 0;
@@ -495,20 +502,28 @@ Slave *Slave::createSlave(const QString &protocol, const QUrl &url, int &error,
         const QStringList args = QStringList() << lib_path << protocol << QLatin1String("") << slaveAddress.toString();
         //qDebug() << "kioslave" << ", " << lib_path << ", " << protocol << ", " << QString() << ", " << slaveAddress;
 
+        // look where libexec path is (can be set in qt.conf)
+        const QString qlibexec = QLibraryInfo::location(QLibraryInfo::LibraryExecutablesPath);
+        // on !win32 we use a kf5 suffix
+        const QString qlibexecKF5 = QDir(qlibexec).filePath(QStringLiteral("kf5"));
+
         // search paths
         const QStringList searchPaths = QStringList()
             << QCoreApplication::applicationDirPath() // then look where our application binary is located
-            << QLibraryInfo::location(QLibraryInfo::LibraryExecutablesPath) // look where libexec path is (can be set in qt.conf)
+            << qlibexec
+            << qlibexecKF5
             << QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5); // look at our installation location
-        const QString kioslaveExecutable = QStandardPaths::findExecutable(QStringLiteral("kioslave"), searchPaths);
+        const QString kioslaveExecutable = QStandardPaths::findExecutable(QStringLiteral("kioslave5"), searchPaths);
         if (kioslaveExecutable.isEmpty()) {
-            error_text = i18n("Can not find 'kioslave' executable at '%1'", searchPaths.join(QStringLiteral(", ")));
+            error_text = i18n("Can not find 'kioslave5' executable at '%1'", searchPaths.join(QLatin1String(", ")));
             error = KIO::ERR_CANNOT_CREATE_SLAVE;
             delete slave;
             return nullptr;
-
         }
-        QProcess::startDetached(kioslaveExecutable, args);
+
+        qint64 pid = 0;
+        QProcess::startDetached(kioslaveExecutable, args, QString(), &pid);
+        slave->setPID(pid);
 
         return slave;
     }
@@ -529,7 +544,7 @@ Slave *Slave::createSlave(const QString &protocol, const QUrl &url, int &error,
         return nullptr;
     }
     slave->setPID(pid);
-    QTimer::singleShot(1000 * SLAVE_CONNECTION_TIMEOUT_MIN, slave, SLOT(timeout()));
+    QTimer::singleShot(1000 * SLAVE_CONNECTION_TIMEOUT_MIN, slave, &Slave::timeout);
     return slave;
 }
 
@@ -558,7 +573,7 @@ Slave *Slave::holdSlave(const QString &protocol, const QUrl &url)
         return nullptr;
     }
     slave->setPID(pid);
-    QTimer::singleShot(1000 * SLAVE_CONNECTION_TIMEOUT_MIN, slave, SLOT(timeout()));
+    QTimer::singleShot(1000 * SLAVE_CONNECTION_TIMEOUT_MIN, slave, &Slave::timeout);
     return slave;
 }
 
