@@ -20,10 +20,9 @@
 #ifndef KFILEPLACESITEM_P_H
 #define KFILEPLACESITEM_P_H
 
-#include <QtCore/QObject>
-#include <QtCore/QPointer>
-#include <QtCore/QModelIndex>
-#include <QtCore/QStringList>
+#include <QObject>
+#include <QPointer>
+#include <QStringList>
 #include <QUrl>
 #include <kbookmark.h>
 #include <solid/device.h>
@@ -51,7 +50,8 @@ public:
         RecentlySavedType,
         SearchForType,
         DevicesType,
-        RemovableDevicesType
+        RemovableDevicesType,
+        TagsType
     };
 
     KFilePlacesItem(KBookmarkManager *manager,
@@ -76,12 +76,14 @@ public:
                                     const QString &iconName,
                                     KFilePlacesItem *after = nullptr);
     static KBookmark createSystemBookmark(KBookmarkManager *manager,
+                                          const QString &translationContext,
                                           const QString &untranslatedLabel,
-                                          const QString &translatedLabel,
                                           const QUrl &url,
                                           const QString &iconName);
     static KBookmark createDeviceBookmark(KBookmarkManager *manager,
                                           const QString &udi);
+    static KBookmark createTagBookmark(KBookmarkManager *manager,
+                                          const QString &tag);
 
 Q_SIGNALS:
     void itemChanged(const QString &id);
