@@ -143,7 +143,7 @@ public:
      */
     void setUrl(const QUrl &url, bool clearforward = true);
 
-#if !defined(KIOFILEWIDGETS_NO_DEPRECATED) && !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(5, 33)
     /**
      * Sets the file to preselect to @p pathOrUrl
      *
@@ -156,7 +156,8 @@ public:
      * @deprecated since 5.33, use setSelectedUrl instead, after ensuring that
      * construct the QUrl correctly (e.g. use fromLocalFile for local paths).
      */
-    KIOFILEWIDGETS_DEPRECATED void setSelection(const QString &pathOrUrl);
+    KIOFILEWIDGETS_DEPRECATED_VERSION(5, 33, "Use KFileWidget::setSelectedUrl(const QUrl &)")
+    void setSelection(const QString &pathOrUrl);
 #endif
 
     /**
@@ -548,7 +549,7 @@ Q_SIGNALS:
     void fileHighlighted(const QUrl &);
 
     /**
-     * Emitted when the user hilights one or more files in multiselection mode.
+     * Emitted when the user highlights one or more files in multiselection mode.
      *
      * Note: fileHighlighted() or fileSelected() are @em not
      * emitted in multiselection mode. You may use selectedItems() to
@@ -613,6 +614,7 @@ private:
     Q_PRIVATE_SLOT(d, void _k_slotIconSizeSliderMoved(int))
     Q_PRIVATE_SLOT(d, void _k_slotIconSizeChanged(int))
     Q_PRIVATE_SLOT(d, void _k_slotViewDoubleClicked(const QModelIndex&))
+    Q_PRIVATE_SLOT(d, void _k_slotViewKeyEnterReturnPressed())
 };
 
 #endif
