@@ -21,6 +21,7 @@
 
 #include "config-kiowidgets.h" // KIO_NO_SOLID
 #include "../core/config-kmountpoint.h" // for HAVE_VOLMGT (yes I cheat a bit)
+#include "kio_widgets_debug.h"
 
 #include "krun.h"
 #include "kautomount.h"
@@ -71,14 +72,14 @@ bool KDesktopFileActions::runWithStartup(const QUrl &u, bool _is_local, const QB
 
     if (u.fileName() == QLatin1String(".directory")) {
         // We cannot execute a .directory file. Open with a text editor instead.
-        return KRun::runUrl(u, QStringLiteral("text/plain"), 0, false /*tempFile*/, false /*runExecutables*/, QString(), asn);
+        return KRun::runUrl(u, QStringLiteral("text/plain"), nullptr, KRun::RunFlags(), QString(), asn);
     }
 
     KDesktopFile cfg(u.toLocalFile());
     if (!cfg.desktopGroup().hasKey("Type")) {
         QString tmp = i18n("The desktop entry file %1 "
                            "has no Type=... entry.", u.toLocalFile());
-        KMessageBox::error(0, tmp);
+        KMessageBox::error(nullptr, tmp);
         return false;
     }
 
@@ -94,7 +95,7 @@ bool KDesktopFileActions::runWithStartup(const QUrl &u, bool _is_local, const QB
     }
 
     QString tmp = i18n("The desktop entry of type\n%1\nis unknown.",  cfg.readType());
-    KMessageBox::error(0, tmp);
+    KMessageBox::error(nullptr, tmp);
 
     return false;
 }
@@ -107,7 +108,7 @@ static bool runFSDevice(const QUrl &_url, const KDesktopFile &cfg, const QByteAr
 
     if (dev.isEmpty()) {
         QString tmp = i18n("The desktop entry file\n%1\nis of type FSDevice but has no Dev=... entry.",  _url.toLocalFile());
-        KMessageBox::error(0, tmp);
+        KMessageBox::error(nullptr, tmp);
         return retval;
     }
 
@@ -116,7 +117,7 @@ static bool runFSDevice(const QUrl &_url, const KDesktopFile &cfg, const QByteAr
     if (mp) {
         const QUrl mpURL = QUrl::fromLocalFile(mp->mountPoint());
         // Open a new window
-        retval = KRun::runUrl(mpURL, QStringLiteral("inode/directory"), 0 /*TODO - window*/, false, true, QString(), asn);
+        retval = KRun::runUrl(mpURL, QStringLiteral("inode/directory"), nullptr /*TODO - window*/, KRun::RunFlags(KRun::RunExecutables), QString(), asn);
     } else {
         KConfigGroup cg = cfg.desktopGroup();
         bool ro = cg.readEntry("ReadOnly", false);
@@ -140,12 +141,11 @@ static bool runApplication(const QUrl &_url, const QString &_serviceFile, const
     if (!s.isValid())
     {
         QString tmp = i18n("The desktop entry file\n%1\nis not valid.", _url.toString());
-        KMessageBox::error(0, tmp);
+        KMessageBox::error(nullptr, tmp);
         return false;
     }
 
-    QList<QUrl> lst;
-    return KRun::runService(s, lst, 0 /*TODO - window*/, false, QString(), asn);
+    return KRun::runApplication(s, QList<QUrl>(), nullptr /*TODO - window*/, KRun::RunFlags{}, QString(), asn);
 }
 
 static bool runLink(const QUrl &_url, const KDesktopFile &cfg, const QByteArray &asn)
@@ -153,12 +153,12 @@ static bool runLink(const QUrl &_url, const KDesktopFile &cfg, const QByteArray
     QString u = cfg.readUrl();
     if (u.isEmpty()) {
         QString tmp = i18n("The desktop entry file\n%1\nis of type Link but has no URL=... entry.",  _url.toString());
-        KMessageBox::error(0, tmp);
+        KMessageBox::error(nullptr, tmp);
         return false;
     }
 
     QUrl url = QUrl::fromUserInput(u);
-    KRun *run = new KRun(url, (QWidget *)0, true, asn);
+    KRun *run = new KRun(url, (QWidget *)nullptr, true, asn);
 
     // X-KDE-LastOpenedWith holds the service desktop entry name that
     // was should be preferred for opening this URL if possible.
@@ -187,7 +187,7 @@ QList<KServiceAction> KDesktopFileActions::builtinServices(const QUrl &_url)
         const QString dev = cfg.readDevice();
         if (dev.isEmpty()) {
             QString tmp = i18n("The desktop entry file\n%1\nis of type FSDevice but has no Dev=... entry.",  _url.toLocalFile());
-            KMessageBox::error(0, tmp);
+            KMessageBox::error(nullptr, tmp);
             return result;
         }
 
@@ -269,7 +269,7 @@ QList<KServiceAction> KDesktopFileActions::userDefinedServices(const KService &s
     QStringList keys;
     const QString actionMenu = service.property(QStringLiteral("X-KDE-GetActionMenu"), QVariant::String).toString();
     if (!actionMenu.isEmpty()) {
-        const QStringList dbuscall = actionMenu.split(QChar(' '));
+        const QStringList dbuscall = actionMenu.split(QLatin1Char(' '));
         if (dbuscall.count() >= 4) {
             const QString &app       = dbuscall.at(0);
             const QString &object    = dbuscall.at(1);
@@ -285,7 +285,7 @@ QList<KServiceAction> KDesktopFileActions::userDefinedServices(const KService &s
                 return result;
             }
         } else {
-            qWarning() << "The desktop file" << service.entryPath()
+            qCWarning(KIO_WIDGETS) << "The desktop file" << service.entryPath()
                        << "has an invalid X-KDE-GetActionMenu entry."
                        << "Syntax is: app object interface function";
         }
@@ -293,10 +293,11 @@ QList<KServiceAction> KDesktopFileActions::userDefinedServices(const KService &s
 
     // Now, either keys is empty (all actions) or it's set to the actions we want
 
-    foreach (const KServiceAction &action, service.actions()) {
+    const QList<KServiceAction> list = service.actions();
+    for (const KServiceAction &action : list) {
         if (keys.isEmpty() || keys.contains(action.name())) {
             const QString exec = action.exec();
-            if (bLocalFiles || exec.contains(QStringLiteral("%U")) || exec.contains(QStringLiteral("%u"))) {
+            if (bLocalFiles || exec.contains(QLatin1String("%U")) || exec.contains(QLatin1String("%u"))) {
                 result.append(action);
             }
         }
@@ -320,7 +321,7 @@ void KDesktopFileActions::executeService(const QList<QUrl> &urls, const KService
             const QString dev = cfg.readDevice();
             if (dev.isEmpty()) {
                 QString tmp = i18n("The desktop entry file\n%1\nis of type FSDevice but has no Dev=... entry.",  path);
-                KMessageBox::error(0, tmp);
+                KMessageBox::error(nullptr, tmp);
                 return;
             }
             KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByDevice(dev);
@@ -386,7 +387,7 @@ void KDesktopFileActions::executeService(const QList<QUrl> &urls, const KService
 #endif
     } else {
         //qDebug() << action.name() << "first url's path=" << urls.first().toLocalFile() << "exec=" << action.exec();
-        KRun::run(action.exec(), urls, 0, action.text(), action.icon());
+        KRun::run(action.exec(), urls, nullptr, action.text(), action.icon());
         // The action may update the desktop file. Example: eject unmounts (#5129).
         org::kde::KDirNotify::emitFilesChanged(urls);
     }
