@@ -19,6 +19,7 @@
 
 #include "kurlrequester.h"
 #include "kio_widgets_debug.h"
+#include "../pathhelpers_p.h"
 
 #include <kcombobox.h>
 #include <kdragwidgetdecorator.h>
@@ -29,24 +30,24 @@
 
 #include <QAction>
 #include <QApplication>
-#include <QDebug>
 #include <QDrag>
 #include <QEvent>
 #include <QKeySequence>
 #include <QHBoxLayout>
 #include <QMimeData>
-#include <QtGlobal>
+#include <QMenu>
+#include <QAction>
 
 class KUrlDragPushButton : public QPushButton
 {
     Q_OBJECT
 public:
-    KUrlDragPushButton(QWidget *parent)
+    explicit KUrlDragPushButton(QWidget *parent)
         : QPushButton(parent)
     {
         new DragDecorator(this);
     }
-    ~KUrlDragPushButton() {}
+    ~KUrlDragPushButton() override {}
 
     void setURL(const QUrl &url)
     {
@@ -58,11 +59,11 @@ private:
     class DragDecorator : public KDragWidgetDecoratorBase
     {
     public:
-        DragDecorator(KUrlDragPushButton *button)
+        explicit DragDecorator(KUrlDragPushButton *button)
             : KDragWidgetDecoratorBase(button), m_button(button) {}
 
     protected:
-        QDrag *dragObject() Q_DECL_OVERRIDE
+        QDrag *dragObject() override
         {
             if (m_button->m_urls.isEmpty()) {
                 return nullptr;
@@ -82,11 +83,12 @@ private:
     QList<QUrl> m_urls;
 };
 
-class KUrlRequester::KUrlRequesterPrivate
+class Q_DECL_HIDDEN KUrlRequester::KUrlRequesterPrivate
 {
 public:
-    KUrlRequesterPrivate(KUrlRequester *parent)
-        : m_parent(parent),
+    explicit KUrlRequesterPrivate(KUrlRequester *parent)
+        : m_fileDialogModeWasDirAndFile(false),
+          m_parent(parent),
           edit(nullptr),
           combo(nullptr),
           fileDialogMode(KFile::File | KFile::ExistingOnly | KFile::LocalOnly),
@@ -123,22 +125,29 @@ public:
 
     void connectSignals(KUrlRequester *receiver)
     {
-        QLineEdit *sender;
         if (combo) {
-            sender = combo->lineEdit();
-        } else {
-            sender = edit;
-        }
-        if (sender) {
-            connect(sender, &QLineEdit::textChanged,
+            connect(combo, &QComboBox::currentTextChanged,
                     receiver, &KUrlRequester::textChanged);
-            connect(sender, &QLineEdit::textEdited,
+            connect(combo, &QComboBox::editTextChanged,
                     receiver, &KUrlRequester::textEdited);
 
-            connect(sender, SIGNAL(returnPressed()),
-                    receiver, SIGNAL(returnPressed()));
-            connect(sender, SIGNAL(returnPressed(QString)),
-                    receiver, SIGNAL(returnPressed(QString)));
+            connect(combo, QOverload<>::of(&KComboBox::returnPressed),
+                    receiver, QOverload<>::of(&KUrlRequester::returnPressed));
+            connect(combo, QOverload<const QString&>::of(&KComboBox::returnPressed),
+                    receiver, QOverload<const QString&>::of(&KUrlRequester::returnPressed));
+        } else if (edit) {
+            connect(edit, &QLineEdit::textChanged,
+                    receiver, &KUrlRequester::textChanged);
+            connect(edit, &QLineEdit::textEdited,
+                    receiver, &KUrlRequester::textEdited);
+
+            connect(edit, QOverload<>::of(&QLineEdit::returnPressed),
+                    receiver, QOverload<>::of(&KUrlRequester::returnPressed));
+
+            if (auto kline = qobject_cast<KLineEdit*>(edit)) {
+                connect(kline, QOverload<const QString&>::of(&KLineEdit::returnPressed),
+                        receiver, QOverload<const QString&>::of(&KUrlRequester::returnPressed));
+            }
         }
     }
 
@@ -189,7 +198,7 @@ public:
         const QUrl enteredUrl = QUrl(enteredPath); // absolute or relative
         if (enteredUrl.isRelative() && !txt.isEmpty()) {
             QUrl finalUrl(m_startDir);
-            finalUrl.setPath(finalUrl.path() + '/' + enteredPath);
+            finalUrl.setPath(concatPaths(finalUrl.path(), enteredPath));
             return finalUrl;
         } else {
             return enteredUrl;
@@ -199,11 +208,12 @@ public:
     static void applyFileMode(QFileDialog *dlg, KFile::Modes m, QFileDialog::AcceptMode acceptMode)
     {
         QFileDialog::FileMode fileMode;
+        bool dirsOnly = false;
         if (m & KFile::Directory) {
             fileMode = QFileDialog::Directory;
             if ((m & KFile::File) == 0 &&
                     (m & KFile::Files) == 0) {
-                dlg->setOption(QFileDialog::ShowDirsOnly, true);
+                dirsOnly = true;
             }
         } else if (m & KFile::Files &&
                    m & KFile::ExistingOnly) {
@@ -217,18 +227,19 @@ public:
 
         dlg->setFileMode(fileMode);
         dlg->setAcceptMode(acceptMode);
+        dlg->setOption(QFileDialog::ShowDirsOnly, dirsOnly);
     }
 
     // Converts from "*.foo *.bar|Comment" to "Comment (*.foo *.bar)"
     QStringList kToQFilters(const QString &filters) const
     {
-        QStringList qFilters = filters.split('\n', QString::SkipEmptyParts);
+        QStringList qFilters = filters.split(QLatin1Char('\n'), QString::SkipEmptyParts);
 
-        for (QStringList::iterator it = qFilters.begin(); it != qFilters.end(); ++it) {
-            int sep = it->indexOf('|');
-            QString globs = it->left(sep);
-            QString desc  = it->mid(sep + 1);
-            *it = QStringLiteral("%1 (%2)").arg(desc, globs);
+        for (QString &qFilter : qFilters) {
+            int sep = qFilter.indexOf(QLatin1Char('|'));
+            const QStringRef globs = qFilter.leftRef(sep);
+            const QStringRef desc  = qFilter.midRef(sep + 1);
+            qFilter = desc + QLatin1String(" (") + globs + QLatin1Char(')');
         }
 
         return qFilters;
@@ -239,6 +250,35 @@ public:
         return QFileDialog::getExistingDirectoryUrl(m_parent, QString(), openUrl, QFileDialog::ShowDirsOnly);
     }
 
+    void createFileDialog()
+    {
+        //Creates the fileDialog if it doesn't exist yet
+        QFileDialog *dlg = m_parent->fileDialog();
+
+        if (!url().isEmpty() && !url().isRelative()) {
+            QUrl u(url());
+            // If we won't be able to list it (e.g. http), then don't try :)
+            if (KProtocolManager::supportsListing(u)) {
+                dlg->selectUrl(u);
+            }
+        } else {
+            dlg->setDirectoryUrl(m_startDir);
+        }
+
+        dlg->setAcceptMode(fileDialogAcceptMode);
+
+        //Update the file dialog window modality
+        if (dlg->windowModality() != fileDialogModality) {
+            dlg->setWindowModality(fileDialogModality);
+        }
+
+        if (fileDialogModality == Qt::NonModal) {
+            dlg->show();
+        } else {
+            dlg->exec();
+        }
+    }
+
     // slots
     void _k_slotUpdateUrl();
     void _k_slotOpenDialog();
@@ -246,7 +286,8 @@ public:
 
     QUrl m_startDir;
     bool m_startDirCustomized;
-    KUrlRequester *m_parent; // TODO: rename to 'q'
+    bool m_fileDialogModeWasDirAndFile;
+    KUrlRequester * const m_parent; // TODO: rename to 'q'
     KLineEdit *edit;
     KComboBox *combo;
     KFile::Modes fileDialogMode;
@@ -268,7 +309,7 @@ KUrlRequester::KUrlRequester(QWidget *editWidget, QWidget *parent)
     d->combo = qobject_cast<KComboBox *>(editWidget);
     d->edit = qobject_cast<KLineEdit *>(editWidget);
     if (d->edit) {
-        d->edit->setClearButtonShown(true);
+        d->edit->setClearButtonEnabled(true);
     }
 
     d->init();
@@ -299,13 +340,13 @@ void KUrlRequester::KUrlRequesterPrivate::init()
 
     if (!combo && !edit) {
         edit = new KLineEdit(m_parent);
-        edit->setClearButtonShown(true);
+        edit->setClearButtonEnabled(true);
     }
 
     QWidget *widget = combo ? static_cast<QWidget *>(combo) : static_cast<QWidget *>(edit);
 
     QHBoxLayout *topLayout = new QHBoxLayout(m_parent);
-    topLayout->setMargin(0);
+    topLayout->setContentsMargins(0, 0, 0, 0);
     topLayout->setSpacing(-1); // use default spacing
     topLayout->addWidget(widget);
 
@@ -343,12 +384,10 @@ void KUrlRequester::setUrl(const QUrl &url)
     d->setText(url.toDisplayString(QUrl::PreferLocalFile));
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 void KUrlRequester::setPath(const QString &path)
 {
     d->setText(path);
 }
-#endif
 
 void KUrlRequester::setText(const QString &text)
 {
@@ -397,10 +436,11 @@ void KUrlRequester::KUrlRequesterPrivate::_k_slotOpenDialog()
             return;
         }
 
-    if (((fileDialogMode & KFile::Directory) && !(fileDialogMode & KFile::File)) ||
-            /* catch possible fileDialog()->setMode( KFile::Directory ) changes */
-            (myFileDialog && (myFileDialog->fileMode() == QFileDialog::Directory &&
-                              myFileDialog->testOption(QFileDialog::ShowDirsOnly)))) {
+    if (!m_fileDialogModeWasDirAndFile &&
+            (((fileDialogMode & KFile::Directory) && !(fileDialogMode & KFile::File)) ||
+             /* catch possible fileDialog()->setMode( KFile::Directory ) changes */
+             (myFileDialog && (myFileDialog->fileMode() == QFileDialog::Directory &&
+                               myFileDialog->testOption(QFileDialog::ShowDirsOnly))))) {
         const QUrl openUrl = (!m_parent->url().isEmpty() && !m_parent->url().isRelative())
                              ? m_parent->url() : m_startDir;
 
@@ -420,31 +460,33 @@ void KUrlRequester::KUrlRequesterPrivate::_k_slotOpenDialog()
     } else {
         emit m_parent->openFileDialog(m_parent);
 
-        //Creates the fileDialog if it doesn't exist yet
-        QFileDialog *dlg = m_parent->fileDialog();
+        if (((fileDialogMode & KFile::Directory) && (fileDialogMode & KFile::File)) || m_fileDialogModeWasDirAndFile) {
+            QMenu *dirOrFileMenu = new QMenu();
+            QAction *fileAction = new QAction(QIcon::fromTheme(QStringLiteral("document-new")), i18n("File"));
+            QAction *dirAction = new QAction(QIcon::fromTheme(QStringLiteral("folder-new")), i18n("Directory"));
+            dirOrFileMenu->addAction(fileAction);
+            dirOrFileMenu->addAction(dirAction);
 
-        if (!url().isEmpty() && !url().isRelative()) {
-            QUrl u(url());
-            // If we won't be able to list it (e.g. http), then don't try :)
-            if (KProtocolManager::supportsListing(u)) {
-                dlg->selectUrl(u);
-            }
-        } else {
-            dlg->setDirectoryUrl(m_startDir);
+            connect(fileAction, &QAction::triggered, [this]() {
+                fileDialogMode = KFile::File;
+                applyFileMode(m_parent->fileDialog(), fileDialogMode, fileDialogAcceptMode);
+                m_fileDialogModeWasDirAndFile = true;
+                createFileDialog();
+            });
+
+            connect(dirAction, &QAction::triggered, [this]() {
+                fileDialogMode = KFile::Directory;
+                applyFileMode(m_parent->fileDialog(), fileDialogMode, fileDialogAcceptMode);
+                m_fileDialogModeWasDirAndFile = true;
+                createFileDialog();
+            });
+
+            dirOrFileMenu->exec(m_parent->mapToGlobal(QPoint(m_parent->width(), m_parent->height())));
+
+            return;
         }
 
-        dlg->setAcceptMode(fileDialogAcceptMode);
-
-        //Update the file dialog window modality
-        if (dlg->windowModality() != fileDialogModality) {
-            dlg->setWindowModality(fileDialogModality);
-        }
-
-        if (fileDialogModality == Qt::NonModal) {
-            dlg->show();
-        } else {
-            dlg->exec();
-        }
+        createFileDialog();
     }
 }
 
@@ -454,7 +496,7 @@ void KUrlRequester::KUrlRequesterPrivate::_k_slotFileDialogAccepted()
         return;
     }
 
-    QUrl newUrl = myFileDialog->selectedUrls().first();
+    const QUrl newUrl = myFileDialog->selectedUrls().constFirst();
     if (newUrl.isValid()) {
         m_parent->setUrl(newUrl);
         emit m_parent->urlSelected(url());
@@ -519,6 +561,7 @@ void KUrlRequester::setMimeTypeFilters(const QStringList &mimeTypes)
     if (d->myFileDialog) {
         d->myFileDialog->setMimeTypeFilters(d->mimeTypeFilters);
     }
+    d->myCompletion->setMimeTypeFilters(d->mimeTypeFilters);
 }
 
 QStringList KUrlRequester::mimeTypeFilters() const
@@ -526,9 +569,15 @@ QStringList KUrlRequester::mimeTypeFilters() const
     return d->mimeTypeFilters;
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 QFileDialog *KUrlRequester::fileDialog() const
 {
+    if (d->myFileDialog &&
+            (   (d->myFileDialog->fileMode() == QFileDialog::Directory && !(d->fileDialogMode & KFile::Directory))
+             || (d->myFileDialog->fileMode() != QFileDialog::Directory &&  (d->fileDialogMode & KFile::Directory)))) {
+        delete d->myFileDialog;
+        d->myFileDialog = nullptr;
+    }
+
     if (!d->myFileDialog) {
         d->myFileDialog = new QFileDialog(window(), windowTitle());
         if (!d->mimeTypeFilters.isEmpty()) {
@@ -545,7 +594,6 @@ QFileDialog *KUrlRequester::fileDialog() const
 
     return d->myFileDialog;
 }
-#endif
 
 void KUrlRequester::clear()
 {
@@ -567,7 +615,7 @@ void KUrlRequester::KUrlRequesterPrivate::_k_slotUpdateUrl()
     const QUrl visibleUrl = url();
     QUrl u = visibleUrl;
     if (visibleUrl.isRelative()) {
-        u = QUrl::fromLocalFile(QDir::currentPath() + '/').resolved(visibleUrl);
+        u = QUrl::fromLocalFile(QDir::currentPath() + QLatin1Char('/')).resolved(visibleUrl);
     }
     myButton->setURL(u);
 }
@@ -594,12 +642,10 @@ KUrlCompletion *KUrlRequester::completionObject() const
     return d->myCompletion;
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 void KUrlRequester::setClickMessage(const QString &msg)
 {
     setPlaceholderText(msg);
 }
-#endif
 
 void KUrlRequester::setPlaceholderText(const QString &msg)
 {
@@ -608,12 +654,10 @@ void KUrlRequester::setPlaceholderText(const QString &msg)
     }
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 QString KUrlRequester::clickMessage() const
 {
     return placeholderText();
 }
-#endif
 
 QString KUrlRequester::placeholderText() const
 {
